/*
 * languess.c -- routines for categorizing text inside PostgreSQL
 *
 * Copyright (C) 2007 Cdric Villemain & Dalibo
 *
 * http://dalibo.com | http://dalibo.org
 *
 * THE BSD LICENSE
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * - Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the
 * distribution.
 *
 * - Neither the name of the WiseGuys Internet B.V. nor the names of
 * its contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "postgres.h"
#include "mb/pg_wchar.h"
#include "utils/elog.h"
#include "utils/builtins.h"

#include "textcat.h"

#ifdef PG_MODULE_MAGIC
PG_MODULE_MAGIC;
#endif

Datum
languess(PG_FUNCTION_ARGS);

PG_FUNCTION_INFO_V1(languess);
Datum
languess(PG_FUNCTION_ARGS) {
  VarChar *str_in     = (VarChar *)PG_GETARG_VARCHAR_P(0);
  int     str_in_size = VARSIZE(str_in) - VARHDRSZ;
  VarChar *return_lang;

  void    *h ;
  char    *buffer_in,*langs;

  pg_verifymbstr(VARDATA(str_in), str_in_size, false);

  buffer_in = DatumGetCString(
                DirectFunctionCall1(textout,
                  PointerGetDatum(str_in)
                )
              );
  PG_FREE_IF_COPY(str_in, 0);

  h = textcat_Init( "/usr/share/postgresql/8.2/contrib/textcat_conf.txt" );
  if (!h) {
    elog(ERROR, "can not open config file : /usr/share/postgresql/8.2/contrib/textcat_conf.txt or one of the fingerprints.");
    PG_RETURN_VARCHAR_P("unable to open one file");
  }

  langs  = textcat_Classify(h,
                            buffer_in,
                            strlen (buffer_in) + 1);

  /* convert string to VarChar for result */
  return_lang = DatumGetVarCharP(
                  DirectFunctionCall2(
                    varcharin,
                    CStringGetDatum(langs),
                    Int32GetDatum(strlen(langs) + VARHDRSZ)
                  )
                );
  textcat_Done(h);

#ifdef LANGUESS_DEBUG
  elog(NOTICE, "languess : %s", return_lang);
#endif /*LANGUESS_DEBUG*/

  PG_RETURN_VARCHAR_P(return_lang);
}
